close all
clear all

% Let user select folder to process
cd('M:\Gap flights budgie\NEW SESSIONS');
[FileName,PathName] = uigetfile('*.mat','Select first file in folder'); % Pick file to get the folder

% Let user choose file with frames to use
[Frames,PathNameFlight] = uigetfile('*.mat','Select FramesAnalysis file'); % Pick file
cd(PathNameFlight) % Open folder
load(Frames); % Load FramesAnalysis file

% Let user choose file with Camera parameters
[NameCalib,PathNameCalib] = uigetfile('*.mat','Select Calibration file'); % Pick file
cd(PathNameCalib)
load(NameCalib)

% Let user choose file with Rotation points
[NameRotpoints,PathNameRot] = uigetfile('*.mat','Select Rotation points file'); % Pick file
cd(PathNameRot)
load(NameRotpoints)

pL1(:,1)=XL(1,1);
pL1(:,2)=YL(1,1);
pR1(:,1)=XR(1,1);
pR1(:,2)=YR(1,1);
PlateXYZ1=triangulate(pL1,pR1,stereoParams);

pL2(:,1)=XL(2,1);
pL2(:,2)=YL(2,1);
pR2(:,1)=XR(2,1);
pR2(:,2)=YR(2,1);
PlateXYZ2=triangulate(pL2,pR2,stereoParams);

pL3(:,1)=XL(3,1);
pL3(:,2)=YL(3,1);
pR3(:,1)=XR(3,1);
pR3(:,2)=YR(3,1);
PlateXYZ3=triangulate(pL3,pR3,stereoParams);

%Rotate the plate points after themselves to show that it looks right
p1new=RotLocalCoord2(PlateXYZ1(1,:),PlateXYZ1(1,:),PlateXYZ2(1,:),PlateXYZ3(1,:));
p2new=RotLocalCoord2(PlateXYZ2(1,:),PlateXYZ1(1,:),PlateXYZ2(1,:),PlateXYZ3(1,:));
p3new=RotLocalCoord2(PlateXYZ3(1,:),PlateXYZ1(1,:),PlateXYZ2(1,:),PlateXYZ3(1,:));

% plot rotated plate points
figure
plot3(p1new(1),p1new(2),p1new(3),'g*')
hold on
plot3(p2new(1),p2new(2),p2new(3),'b*')
plot3(p3new(1),p3new(2),p3new(3),'r*')
xlabel('X')
ylabel('Y')
zlabel('Z')
axis equal

clear XL YL XR YR pL1 pR1

% Get list of files in folder
cd(PathName) % Open folder with files
Files=dir('*.mat');


for i=1:length(Files)
start=FramesAnalysis(i,4);
stop=FramesAnalysis(i,5);
% **** Load Data ****
load(Files(i).name)
mkdir(Files(i).name(1:end-4)); % Make a new subfolder for the current flight
cd(Files(i).name(1:end-4)); %Go into new folder
pL1(:,1)=XL(1,start:stop);
pL1(:,2)=YL(1,start:stop);
pR1(:,1)=XR(1,start:stop);
pR1(:,2)=YR(1,start:stop);

% **** Calculate 3D ****
[XYZ1 error]=triangulate(pL1,pR1,stereoParams);

XYZ1_old=XYZ1; %store the original points before rotation

% plot original points before rotation
figure
plot3(XYZ1_old(:,1),XYZ1_old(:,2),XYZ1_old(:,3),'r')
xlabel('X')
ylabel('Y')
zlabel('Z')
axis equal
savefig('Raw unrotated')
saveas(gcf,'Raw unrotated.png')

XYZ1=RotLocalCoord2(XYZ1,PlateXYZ1(1,:),PlateXYZ2(1,:),PlateXYZ3(1,:)); % rotate the points so that the plate points define the coordinate system.
XYZ1(:,3)=-XYZ1(:,3); % Z-axis needs to be inverted

% **** Smoothing ****
t = 0:1/120:(size(pL1,1)-1)/120; % t for all timestep
t=t';
t_rev=-flipud(t); % time when 0 set to when the bird has reached the gap

XYZ1s(:,1)=csaps(t,XYZ1(:,1),1-1e-7,t(:));
XYZ1s(:,2)=csaps(t,XYZ1(:,2),1-1e-7,t(:));
XYZ1s(:,3)=csaps(t,XYZ1(:,3),1-1e-7,t(:));

% **** Plot each axis with and without smoothing ****
figure
subplot(3,1,1)
plot(t,XYZ1(:,1),'r')
hold on
plot(t,XYZ1s(:,1),'g')
title('X')

subplot(3,1,2)
plot(t,XYZ1(:,2),'r')
hold on
plot(t,XYZ1s(:,2),'g')
title('Y')

subplot(3,1,3)
plot(t,XYZ1(:,3),'r')
hold on
plot(t,XYZ1s(:,3),'g')
title('Z')
savefig('Raw and smoothed')
saveas(gcf,'Raw and smoothed.png')

% **** Plot 3D trajectory with and without smoothing ****
figure
plot3(XYZ1(:,1),XYZ1(:,2),XYZ1(:,3),'r')
xlabel('X')
ylabel('Y')
zlabel('Z')
axis equal
hold on
plot3(XYZ1s(:,1),XYZ1s(:,2),XYZ1s(:,3),'g')
plot3(XYZ1(1,1),XYZ1(1,2),XYZ1(1,3),'g*') % plot first point in green
plot3(XYZ1(end,1),XYZ1(end,2),XYZ1(end,3),'r*') % plot last point in red
savefig('Raw and smoothed 3D')
saveas(gcf,'Raw and smoothed 3D.png')

figure %with plate points for check of rotation
plot3(XYZ1(:,1),XYZ1(:,2),XYZ1(:,3),'r')
xlabel('X')
ylabel('Y')
zlabel('Z')
axis equal
hold on
plot3(XYZ1s(:,1),XYZ1s(:,2),XYZ1s(:,3),'g')
plot3(XYZ1(1,1),XYZ1(1,2),XYZ1(1,3),'g*') % plot first point in green
plot3(XYZ1(end,1),XYZ1(end,2),XYZ1(end,3),'r*') % plot last point in red
hold on
plot3(p1new(1),p1new(2),p1new(3),'g*')
plot3(p2new(1),p2new(2),p2new(3),'b*')
plot3(p3new(1),p3new(2),p3new(3),'r*')
savefig('Raw and smoothed 3D with plate')
saveas(gcf,'Raw and smoothed 3D with plate.png')


% **** Calculate speed ****
spX1=csaps(t,XYZ1(:,1),1-1e-7); %fit function to X points
DerX1=fnder(spX1); % first derivative
VelX1=fnval(DerX1,t); %extract values
spY1=csaps(t,XYZ1(:,2),1-1e-7); %fit function to Y points
DerY1=fnder(spY1); % first derivative
VelY1=fnval(DerY1,t); %extract values
spZ1=csaps(t,XYZ1(:,3),1-1e-7); %fit function to Z points
DerZ1=fnder(spZ1); % first derivative
VelZ1=fnval(DerZ1,t); %extract values

VelTOT=(sqrt((VelX1.^2)+(VelY1.^2)+(VelZ1.^2)))./1000; % total speed in m/s both horizontal and vertical together

Flightspeed_TOT=mean(VelTOT) % mean total flight speed over the sequence

VelXY=(sqrt((VelX1.^2)+(VelY1.^2)))./1000; % XY speed in m/s
Flightspeed_H=mean(VelXY) % mean horisontal speed over the sequence

VelZ=VelZ1./1000; % Z speed in m/s
Flightspeed_V=mean(VelZ1./1000) % mean vertical speed over the sequence

% **** Plot velocities ****
figure
subplot(5,1,1)
plot(t,VelX1./1000)
title('Flight direction (X)')

subplot(5,1,2)
plot(t,VelY1./1000)
title('Lateral (Y)')

subplot (5,1,3)
plot(t,VelXY)
title('Horisontal Tot (XY)')

subplot(5,1,4)
plot(t,VelZ)
title('Vertical (Z)')

subplot(5,1,5)
plot(t,VelTOT)
title('Total (XYZ)')
savefig('Velocities')
saveas(gcf,'Velocities.png')

% **** Dummy check using first and last frame ****

diff=XYZ1(1,:)-XYZ1(end,:); % 3D distance from first to last position
diff2=diff.*diff; 

diffsum=sum(diff2,2);

dist=(sqrt(diffsum))/1000;
time=(1/120)*(stop-start);

Flightspeed_check=dist/time

Error_avg(i)=mean(error)

% **** Saving results ****

save ([savename(1:end-4) '_Everything']); % Save everything
save ([savename(1:end-4) '_results'],'t','t_rev','XYZ1','XYZ1s','error','VelX1','VelY1','VelZ1','VelTOT','VelXY','VelZ','Flightspeed_TOT','Flightspeed_H','Flightspeed_V','Flightspeed_check','stereoParams','pL1','pR1'); % Save relevant results

% Construct matrix for xls-output. Trajectory (smoothed)
trajxls(:,1)=t;
trajxls(:,2)=t_rev;
trajxls(:,3)=XYZ1s(:,1);
trajxls(:,4)=XYZ1s(:,2);
trajxls(:,5)=XYZ1s(:,3);
xlswrite([savename(1:end-4) '_trajectory'],trajxls); % save to xls-file

% Construct matrix for xls-output. Speeds
speedxls(:,1)=t;
speedxls(:,2)=t_rev;
speedxls(:,3)=VelX1./1000;
speedxls(:,4)=VelY1./1000;
speedxls(:,5)=VelZ1./1000;
speedxls(:,6)=VelTOT;
speedxls(:,7)=VelTOT;
speedxls(:,8)=VelTOT;
xlswrite([savename(1:end-4) '_speeds'],speedxls); % save to xls-file

totspeedxls(i,:)={savename(1:end-4) Flightspeed_TOT Flightspeed_H Flightspeed_V Flightspeed_check}; % Save mean speed and dummy check for each file in the folder for xls

clear pL1 pR1 XYZ1 XYZ1s trajxls speedxls
close all
cd('..'); % Go up one level to main folder before starting over
end

xlswrite([savename(1:2) '_TOTspeeds'],totspeedxls); % Save all mean speeds to xls
save([savename(1:2) '_AvgError'],'Error_avg');
Overall_error=mean(Error_avg)
